@@ -22,16 +22,12 @@
 
 #include "types.h"
 
-#define CAST5_BLOCKSIZE 8
-
-typedef struct {
-    u32  Km[16];
-    byte Kr[16];
-} CAST5_context;
-
-void cast5_setkey( CAST5_context *c, byte *key, unsigned keylen );
-void cast5_encrypt_block( CAST5_context *bc, byte *outbuf, byte *inbuf );
-void cast5_decrypt_block( CAST5_context *bc, byte *outbuf, byte *inbuf );
-
+const char *
+cast5_get_info( int algo, size_t *keylen,
+		   size_t *blocksize, size_t *contextsize,
+		   void (**setkey)( void *c, byte *key, unsigned keylen ),
+		   void (**encrypt)( void *c, byte *outbuf, byte *inbuf ),
+		   void (**decrypt)( void *c, byte *outbuf, byte *inbuf )
+		 );
 
 #endif /*G10_CAST5_H*/
