@@ -27,7 +27,7 @@
 #define _g10lib_INTERNAL 1
 #include "g10lib.h"
 
-const char *g10c_revision_string(int dummy) { return "$Revision: 1.2 $"; }
+const char *g10c_revision_string(int dummy) { return "$Revision: 1.4 $"; }
 
 MPI
 g10c_generate_secret_prime( unsigned nbits )
@@ -35,10 +35,10 @@ g10c_generate_secret_prime( unsigned nbits )
     return generate_secret_prime( nbits );
 }
 
-byte
-g10c_get_random_byte( int level )
+
+char *
+g10c_get_random_bits( unsigned nbits, int level, int secure )
 {
-    return get_random_byte( level );
+    return (char*)get_random_bits( nbits, level, secure );
 }
 
-
