@@ -30,6 +30,7 @@
 #include "util.h"
 #include "cipher.h"
 #include "memory.h"
+#include "options.h"
 
 void
 free_symkey_enc( PKT_symkey_enc *enc )
@@ -55,7 +56,7 @@ void
 free_seckey_enc( PKT_signature *sig )
 {
     int n, i;
-    n = pubkey_get_nenc( sig->pubkey_algo );
+    n = pubkey_get_nsig( sig->pubkey_algo );
     if( !n ) {
 	m_free(sig->data[0]);
 	sig->data[0] = NULL;
@@ -82,6 +83,10 @@ release_public_key_parts( PKT_public_key *pk )
 	mpi_free( pk->pkey[i] );
 	pk->pkey[i] = NULL;
     }
+    if( pk->namehash ) {
+	m_free(pk->namehash);
+	pk->namehash = NULL;
+    }
 }
 
 
@@ -107,15 +112,38 @@ cp_fake_data( MPI a )
     return d;
 }
 
+static void *
+cp_data_block( byte *s )
+{
+    byte *d;
+    u16 len;
+
+    if( !s )
+	return NULL;
+    len = (s[0] << 8) | s[1];
+    d = m_alloc( len+2 );
+    memcpy(d, s, len+2);
+    return d;
+}
+
 
 PKT_public_key *
-copy_public_key( PKT_public_key *d, PKT_public_key *s )
+copy_public_key_new_namehash( PKT_public_key *d, PKT_public_key *s,
+			      const byte *namehash )
 {
     int n, i;
 
     if( !d )
 	d = m_alloc(sizeof *d);
     memcpy( d, s, sizeof *d );
+    if( namehash ) {
+	d->namehash = m_alloc( 20 );
+	memcpy(d->namehash, namehash, 20 );
+    }
+    else if( s->namehash ) {
+	d->namehash = m_alloc( 20 );
+	memcpy(d->namehash, s->namehash, 20 );
+    }
     n = pubkey_get_npkey( s->pubkey_algo );
     if( !n )
 	d->pkey[0] = cp_fake_data(s->pkey[0]);
@@ -126,6 +154,44 @@ copy_public_key( PKT_public_key *d, PKT_public_key *s )
     return d;
 }
 
+PKT_public_key *
+copy_public_key( PKT_public_key *d, PKT_public_key *s )
+{
+   return copy_public_key_new_namehash( d, s, NULL );
+}
+
+PKT_signature *
+copy_signature( PKT_signature *d, PKT_signature *s )
+{
+    int n, i;
+
+    if( !d )
+	d = m_alloc(sizeof *d);
+    memcpy( d, s, sizeof *d );
+    n = pubkey_get_nsig( s->pubkey_algo );
+    if( !n )
+	d->data[0] = cp_fake_data(s->data[0]);
+    else {
+	for(i=0; i < n; i++ )
+	    d->data[i] = mpi_copy( s->data[i] );
+    }
+    d->hashed_data = cp_data_block(s->hashed_data);
+    d->unhashed_data = cp_data_block(s->unhashed_data);
+    return d;
+}
+
+
+PKT_user_id *
+copy_user_id( PKT_user_id *d, PKT_user_id *s )
+{
+    if( !d )
+	d = m_alloc(sizeof *d + s->len - 1 );
+    memcpy( d, s, sizeof *d + s->len - 1 );
+    return d;
+}
+
+
+
 void
 release_secret_key_parts( PKT_secret_key *sk )
 {
