@@ -22,10 +22,15 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #if defined(__linux__) && defined(__alpha__)
   #include <asm/sysinfo.h>
   #include <asm/unistd.h>
 #endif
+#ifdef HAVE_SETRLIMIT
+  #include <sys/time.h>
+  #include <sys/resource.h>
+#endif
 #include "util.h"
 #include "main.h"
 #include "options.h"
@@ -70,13 +75,30 @@ trap_unaligned(void)
 #endif
 
 
+void
+disable_core_dumps()
+{
+  #ifdef HAVE_SETRLIMIT
+    struct rlimit limit;
+
+    limit.rlim_cur = 0;
+    limit.rlim_max = 0;
+    if( setrlimit( RLIMIT_CORE, &limit ) )
+	log_fatal("can't disable core dumps: %s\n", strerror(errno) );
+  #else
+    log_info("WARNING: Program may create a core file!\n");
+  #endif
+}
+
+
+
 u16
 checksum_u16( unsigned n )
 {
     u16 a;
 
     a  = (n >> 8) & 0xff;
-    if( opt.emulate_bugs & 1 ) {
+    if( opt.emulate_bugs & EMUBUG_GPGCHKSUM ) {
        a |= n & 0xff;
        log_debug("csum_u16 emulated for n=%u\n", n);
     }
@@ -120,7 +142,7 @@ checksum_mpi( MPI a )
      * this stored value if it is still available.
      */
 
-    if( opt.emulate_bugs & 1 )
+    if( opt.emulate_bugs & EMUBUG_GPGCHKSUM )
 	nbits = 0;
     else
 	nbits = mpi_get_nbit_info(a);
