@@ -52,6 +52,8 @@ struct {
     int rfc1991;
     unsigned emulate_bugs; /* bug emulation flags EMUBUG_xxxx */
     int shm_coprocess;
+    const char *set_filename;
+    const char *comment_string;
 } opt;
 
 
