@@ -112,6 +112,7 @@ typedef struct {
     byte    pubkey_usage;   /* for now only used to pass it to getkey() */
     ulong   local_id;	    /* internal use, valid if > 0 */
     u32     keyid[2];	    /* calculated by keyid_from_pk() */
+    byte    *namehash;	    /* if != NULL: found by this name */
     MPI     pkey[PUBKEY_MAX_NPKEY];
 } PKT_public_key;
 
@@ -222,7 +223,7 @@ typedef enum {
 
 /*-- mainproc.c --*/
 int proc_packets( IOBUF a );
-int proc_signature_packets( IOBUF a, STRLIST signedfiles );
+int proc_signature_packets( IOBUF a, STRLIST signedfiles, const char *sigfile );
 int proc_encryption_packets( IOBUF a );
 int list_packets( IOBUF a );
 
@@ -235,6 +236,8 @@ int copy_some_packets( IOBUF inp, IOBUF out, ulong stopoff );
 int skip_some_packets( IOBUF inp, unsigned n );
 const byte *parse_sig_subpkt( const byte *buffer,
 			      sigsubpkttype_t reqtype, size_t *ret_n );
+const byte *parse_sig_subpkt2( PKT_signature *sig,
+			       sigsubpkttype_t reqtype, size_t *ret_n );
 
 /*-- build-packet.c --*/
 int build_packet( IOBUF inp, PACKET *pkt );
@@ -257,8 +260,13 @@ void free_user_id( PKT_user_id *uid );
 void free_comment( PKT_comment *rem );
 void free_packet( PACKET *pkt );
 PKT_public_key *copy_public_key( PKT_public_key *d, PKT_public_key *s );
+PKT_public_key *copy_public_key_new_namehash( PKT_public_key *d,
+					      PKT_public_key *s,
+					      const byte *namehash );
 PKT_secret_key *copy_secret_key( PKT_secret_key *d, PKT_secret_key *s );
-int cmp_public_keys( PKT_public_key *a, PKT_public_key *b );
+PKT_signature *copy_signature( PKT_signature *d, PKT_signature *s );
+PKT_user_id *copy_user_id( PKT_user_id *d, PKT_user_id *s );
+int cmp_public_keys( PKT_public_key *d, PKT_public_key *s );
 int cmp_public_secret_key( PKT_public_key *pk, PKT_secret_key *sk );
 int cmp_user_ids( PKT_user_id *a, PKT_user_id *b );
 
@@ -298,4 +306,7 @@ int make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
 			int (*mksubpkt)(PKT_signature *, void *),
 			void *opaque  );
 
+/*-- keygen.c --*/
+PKT_user_id *generate_user_id(void);
+
 #endif /*G10_PACKET_H*/
